@@ -762,6 +762,7 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
           debug_print ("CLIENT: WaitingForAgreeUnixFD, read=`%s'", line);
           if (g_strcmp0 (line, "AGREE_UNIX_FD") == 0)
             {
+              g_free (line);
               negotiated_capabilities |= G_DBUS_CAPABILITY_FLAGS_UNIX_FD_PASSING;
               s = "BEGIN\r\n";
               debug_print ("CLIENT: writing `%s'", s);
@@ -804,7 +805,7 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
             {
               gchar *encoded;
               gchar *decoded_data;
-              gsize decoded_data_len;
+              gsize decoded_data_len = 0;
 
               encoded = g_strdup (line + 5);
               g_free (line);
@@ -996,6 +997,7 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
     {
       local_error = NULL;
       byte = g_data_input_stream_read_byte (dis, cancellable, &local_error);
+      byte = byte; /* To avoid -Wunused-but-set-variable */
       if (local_error != NULL)
         {
           g_propagate_error (error, local_error);
@@ -1229,7 +1231,7 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
             {
               gchar *encoded;
               gchar *decoded_data;
-              gsize decoded_data_len;
+              gsize decoded_data_len = 0;
 
               encoded = g_strdup (line + 5);
               g_free (line);
